#include <IOKit/IOLib.h>
#include <IOKit/IOBufferMemoryDescriptor.h>
#include <IOKit/pci/IOPCIDevice.h>
#include "ATY_Init.h"
//#include "IONDRVLibraries.h"
#include "rhd_rec.h"
#include "DEBUG.h"

#define RHD_FB_BAR   0
#define RHD_MMIO_BAR 2

#define RHD_VBIOS_BASE 0xC0000
#define RHD_VBIOS_SIZE 0x10000

extern "C" int rhdAtomInit(RHDPtr rhdPtr, unsigned char *BIOSbase, UInt32 BIOSImageSize);

#undef super
#define super IOFramebuffer

OSDefineMetaClassAndStructors(ATY_Init, IONDRVFramebuffer)

IOService * ATY_Init::probe(IOService * provider, SInt32 * score) {
	bool unposted = true;	//do you have a way to determine this?

	IOPCIDevice *device = NULL;
	IOMemoryMap *IOmap = NULL;
	IOMemoryMap *FBmap = NULL;
    IOMemoryDescriptor * mem;
	
	RHDRec rhdRec;
	const void *ptr = NULL;
	UInt32 BIOSImageSize = 0;
	unsigned char *BIOSbase = NULL;

	if (!unposted) return NULL;
	do {	
		device = OSDynamicCast(IOPCIDevice, provider);
		if (device == NULL) device = OSDynamicCast(IOPCIDevice, provider->getProvider());
		if (device == NULL) {
			LOG("Start IOPCIDevice not found\n");
			break;
		}
		device->setMemoryEnable(true);
		
		//get the IO Mapped address
		IOmap = device->mapDeviceMemoryWithRegister(kIOPCIConfigBaseAddress2);
		if (IOmap == NULL) {
			LOG("Map IO memory failed\n");
			break;
		}
		rhdRec.IObase = (void *)IOmap->getVirtualAddress();
		
		//get the FB Mapped address
		FBmap = device->mapDeviceMemoryWithRegister(kIOPCIConfigBaseAddress0);
		if (FBmap == NULL) {
			LOG("Map Framebuffer memory failed\n");
			break;
		}
		rhdRec.FBbase = (void *)FBmap->getVirtualAddress();
		
		//get the RegEntryID, needed for PCI Config RW
		MAKE_REG_ENTRY(&rhdRec.device, device);

		//get card chipType (deviceID)
		rhdRec.chipType = device->configRead16(kIOPCIConfigDeviceID);
		
		//get card chipSet
		int i = 0;
		bool found = false;
		while ( RHDPCIchipsets[i].numChipset > 0 )
		{
			if (rhdRec.chipType == RHDPCIchipsets[i].PCIid)
			{
				rhdRec.ChipSet = RHDPCIchipsets[i].numChipset;
				found = true;
				break;
			}
			i++;
		}
		if (!found) {
			LOG1("unknown chipType with deviceID: %04x\n", rhdRec.chipType);
			break;
		}
		
		//get the BIOS ROM
		OSData *data = OSDynamicCast(OSData, provider->getProperty("ATY,bin_image"));
		if (data != NULL) {
			BIOSImageSize = data->getLength();
			if (BIOSImageSize > RHD_VBIOS_SIZE) {
				LOG1("Invalid BIOS length: %05x\n", BIOSImageSize);
				break;
			}
			ptr = data->getBytesNoCopy();
			if ((* (UInt16 *)ptr) != 0xAA55) {
				LOG1("Wrong BIOS signature: %04x\n", (* (UInt16 *)ptr));
				break;
			}
			BIOSbase = (unsigned char *) IOMalloc(BIOSImageSize);
			if (BIOSbase == NULL) {
				LOG("Allocate memory for BIOS copy failed\n");
				break;
			}
			for (i = 0;i < BIOSImageSize;i++) BIOSbase[i] = (* ((unsigned char *)(ptr) + i));
		} else {
			LOG("Read \"ATY,bin_image\" error, trying to get BIOS from legacy address\n");
			mem = IOMemoryDescriptor::withPhysicalAddress ( (IOPhysicalAddress)RHD_VBIOS_BASE, RHD_VBIOS_SIZE, kIODirectionOut );
			if (mem == NULL) {
				LOG("failed to get BIOS from legacy address\n");
				break;
			}
			BIOSImageSize = RHD_VBIOS_SIZE;
			BIOSbase = (unsigned char *) IOMalloc(BIOSImageSize);
			if (BIOSbase == NULL) {
				LOG("Allocate memory for BIOS copy failed\n");
				break;
			}
			mem->prepare(kIODirectionOut);
			if (mem->readBytes(0, (void *)BIOSbase, RHD_VBIOS_SIZE) != RHD_VBIOS_SIZE) {
				LOG("Copy BIOS from leagcy address failed\n");
				break;
			}
			mem->complete(kIODirectionOut);
			if ((* (UInt16 *)BIOSbase) != 0xAA55) {
				LOG1("Wrong BIOS signature: %04x\n", (* (UInt16 *)BIOSbase));
				break;
			}
		}
		
		//initialize the card now
		if (rhdAtomInit(&rhdRec, BIOSbase, BIOSImageSize)) LOG("Card initialize completed\n");
		else LOG("Card initialize failed\n");
	} while (0);
	
	if (IOmap != NULL) IOmap->release();
	if (FBmap != NULL) FBmap->release();
	if (BIOSbase != NULL) IOFree(BIOSbase, BIOSImageSize);
	
	return NULL;
}

