/*
 *  rhd_init.c
 *  ATY_Init
 *
 *  Created by Dong Luo on 3/20/09.
 *  Copyright 2009 Boston University. All rights reserved.
 *
 */
#include <IOKit/IOLib.h>
#include <IOKit/ndrvsupport/IOMacOSTypes.h>
#include <string.h>
#include "DEBUG.h"
#include "rhd_init.h"
#include "CD_Common_Types.h"
#include "atombios.h"
#include "rhd_atomwrapper.h"
#include "rhd_rec.h"
#include "rhd_regs.h"

#ifndef __IONDRVLIBRARIES__	//directly include IONDRVLibraries.h cause some problems, switched to to declare them here

OSErr ExpMgrConfigReadLong(
						   RegEntryIDPtr    node,
						   LogicalAddress   configAddr,
						   UInt32 *         valuePtr);

OSErr ExpMgrConfigWriteLong(
							RegEntryIDPtr    node,
							LogicalAddress   configAddr,
							UInt32           value);

OSErr ExpMgrConfigReadWord(
						   RegEntryIDPtr    node,
						   LogicalAddress   configAddr,
						   UInt16 *         valuePtr);

OSErr ExpMgrConfigWriteWord(
							RegEntryIDPtr    node,
							LogicalAddress   configAddr,
							UInt16           value);

OSErr ExpMgrConfigReadByte(
						   RegEntryIDPtr    node,
						   LogicalAddress   configAddr,
						   UInt8 *          valuePtr);

OSErr ExpMgrConfigWriteByte(
							RegEntryIDPtr    node,
							LogicalAddress   configAddr,
							UInt8            value);

OSErr ExpMgrIOReadLong(
					   RegEntryIDPtr    node,
					   LogicalAddress   ioAddr,
					   UInt32 *         valuePtr);
LogicalAddress PoolAllocateResident(ByteCount byteSize, Boolean clear);

OSStatus PoolDeallocate( LogicalAddress address );

#endif

#define ATOM_MAX_SUPPORTED_DEVICE_INFO_HD (ATOM_DEVICE_RESERVEDF_INDEX+1)
typedef struct _ATOM_SUPPORTED_DEVICES_INFO_HD
	{
		ATOM_COMMON_TABLE_HEADER      sHeader;
		USHORT                        usDeviceSupport;
		ATOM_CONNECTOR_INFO_I2C       asConnInfo[ATOM_MAX_SUPPORTED_DEVICE_INFO_HD];
		ATOM_CONNECTOR_INC_SRC_BITMAP asIntSrcInfo[ATOM_MAX_SUPPORTED_DEVICE_INFO_HD];
	} ATOM_SUPPORTED_DEVICES_INFO_HD;

typedef struct _atomDataTables
	{
		unsigned char                       *UtilityPipeLine;
		ATOM_MULTIMEDIA_CAPABILITY_INFO     *MultimediaCapabilityInfo;
		ATOM_MULTIMEDIA_CONFIG_INFO         *MultimediaConfigInfo;
		ATOM_STANDARD_VESA_TIMING           *StandardVESA_Timing;
		union {
			void                            *base;
			ATOM_FIRMWARE_INFO              *FirmwareInfo;
			ATOM_FIRMWARE_INFO_V1_2         *FirmwareInfo_V_1_2;
			ATOM_FIRMWARE_INFO_V1_3         *FirmwareInfo_V_1_3;
			ATOM_FIRMWARE_INFO_V1_4         *FirmwareInfo_V_1_4;
		} FirmwareInfo;
		ATOM_DAC_INFO                       *DAC_Info;
		union {
			void                            *base;
			ATOM_LVDS_INFO                  *LVDS_Info;
			ATOM_LVDS_INFO_V12              *LVDS_Info_v12;
		} LVDS_Info;
		ATOM_TMDS_INFO                      *TMDS_Info;
		ATOM_ANALOG_TV_INFO                 *AnalogTV_Info;
		union {
			void                            *base;
			ATOM_SUPPORTED_DEVICES_INFO     *SupportedDevicesInfo;
			ATOM_SUPPORTED_DEVICES_INFO_2   *SupportedDevicesInfo_2;
			ATOM_SUPPORTED_DEVICES_INFO_2d1 *SupportedDevicesInfo_2d1;
			ATOM_SUPPORTED_DEVICES_INFO_HD  *SupportedDevicesInfo_HD;
		} SupportedDevicesInfo;
		ATOM_GPIO_I2C_INFO                  *GPIO_I2C_Info;
		ATOM_VRAM_USAGE_BY_FIRMWARE         *VRAM_UsageByFirmware;
		ATOM_GPIO_PIN_LUT                   *GPIO_Pin_LUT;
		ATOM_VESA_TO_INTENAL_MODE_LUT       *VESA_ToInternalModeLUT;
		union {
			void                            *base;
			ATOM_COMPONENT_VIDEO_INFO       *ComponentVideoInfo;
			ATOM_COMPONENT_VIDEO_INFO_V21   *ComponentVideoInfo_v21;
		} ComponentVideoInfo;
		/**/unsigned char                       *PowerPlayInfo;
		COMPASSIONATE_DATA                  *CompassionateData;
		ATOM_DISPLAY_DEVICE_PRIORITY_INFO   *SaveRestoreInfo;
		/**/unsigned char                       *PPLL_SS_Info;
		ATOM_OEM_INFO                       *OemInfo;
		ATOM_XTMDS_INFO                     *XTMDS_Info;
		ATOM_ASIC_MVDD_INFO                 *MclkSS_Info;
		ATOM_OBJECT_HEADER                  *Object_Header;
		INDIRECT_IO_ACCESS                  *IndirectIOAccess;
		ATOM_MC_INIT_PARAM_TABLE            *MC_InitParameter;
		/**/unsigned char                       *ASIC_VDDC_Info;
		ATOM_ASIC_INTERNAL_SS_INFO          *ASIC_InternalSS_Info;
		/**/unsigned char                       *TV_VideoMode;
		union {
			void                            *base;
			ATOM_VRAM_INFO_V2               *VRAM_Info_v2;
			ATOM_VRAM_INFO_V3               *VRAM_Info_v3;
		} VRAM_Info;
		ATOM_MEMORY_TRAINING_INFO           *MemoryTrainingInfo;
		union {
			void                            *base;
			ATOM_INTEGRATED_SYSTEM_INFO     *IntegratedSystemInfo;
			ATOM_INTEGRATED_SYSTEM_INFO_V2  *IntegratedSystemInfo_v2;
		} IntegratedSystemInfo;
		ATOM_ASIC_PROFILING_INFO            *ASIC_ProfilingInfo;
		ATOM_VOLTAGE_OBJECT_INFO            *VoltageObjectInfo;
		ATOM_POWER_SOURCE_INFO              *PowerSourceInfo;
	} atomDataTables, *atomDataTablesPtr;

enum atomRegisterType {
    atomRegisterMMIO,
    atomRegisterMC,
    atomRegisterPLL,
    atomRegisterPCICFG
};

struct atomSaveListRecord
{
    /* header */
    int Length;
    int Last;
    struct atomRegisterList{
		enum atomRegisterType Type;
		CARD32 Address;
		CARD32 Value;
    } RegisterList[1];
};

struct atomSaveListObject
{
    struct atomSaveListObject *next;
    struct atomSaveListRecord **SaveList;
};

typedef struct _atomBiosHandle {
    RHDPtr rhdPtr;
    unsigned char *BIOSBase;
    atomDataTablesPtr atomDataPtr;
    pointer *scratchBase;
    CARD32 fbBase;
    unsigned int BIOSImageSize;
    unsigned char *codeTable;
    struct atomSaveListRecord **SaveList;
    struct atomSaveListObject *SaveListObjects;
} atomBiosHandleRec;

static int
rhdAtomAnalyzeCommonHdr(ATOM_COMMON_TABLE_HEADER *hdr)
{
    if (hdr->usStructureSize == 0xaa55)
        return FALSE;
	
    return TRUE;
}

static int
rhdAtomAnalyzeRomHdr(unsigned char *rombase,
					 ATOM_ROM_HEADER *hdr,
					 unsigned int *data_offset, unsigned int *code_table)
{
    if (!rhdAtomAnalyzeCommonHdr(&hdr->sHeader)) {
        return FALSE;
    }
    LOG2("\tSubsystemVendorID: 0x%4.4x SubsystemID: 0x%4.4x\n",
		 hdr->usSubsystemVendorID,hdr->usSubsystemID);
    LOG1("\tIOBaseAddress: 0x%4.4x\n",hdr->usIoBaseAddress);
    LOG1("\tFilename: %s\n",rombase + hdr->usConfigFilenameOffset);
    LOG1("\tBIOS Bootup Message: %s\n",
		 rombase + hdr->usBIOS_BootupMessageOffset);
	
    *data_offset = hdr->usMasterDataTableOffset;
    *code_table = hdr->usMasterCommandTableOffset;
	
    return TRUE;
}

static int
rhdAtomAnalyzeRomDataTable(unsigned char *base, int offset,
						   void *ptr,unsigned short *size)
{
    ATOM_COMMON_TABLE_HEADER *table = (ATOM_COMMON_TABLE_HEADER *)
	(base + offset);
	
	if (!*size || !rhdAtomAnalyzeCommonHdr(table)) {
		if (*size) *size -= 2;
		*(void **)ptr = NULL;
		return FALSE;
	}
	*size -= 2;
	*(void **)ptr = (void *)(table);
	return TRUE;
}

static Bool
rhdAtomGetTableRevisionAndSize(ATOM_COMMON_TABLE_HEADER *hdr,
							   CARD8 *contentRev,
							   CARD8 *formatRev,
							   unsigned short *size)
{
    if (!hdr)
        return FALSE;
	
    if (contentRev) *contentRev = hdr->ucTableContentRevision;
    if (formatRev) *formatRev = hdr->ucTableFormatRevision;
    if (size) *size = (short)hdr->usStructureSize
		- sizeof(ATOM_COMMON_TABLE_HEADER);
	
    return TRUE;
}

static Bool
rhdAtomAnalyzeMasterDataTable(unsigned char *base,
							  ATOM_MASTER_DATA_TABLE *table,
							  atomDataTablesPtr data)
{
    ATOM_MASTER_LIST_OF_DATA_TABLES *data_table =
	&table->ListOfDataTables;
    unsigned short size;
	
    if (!rhdAtomAnalyzeCommonHdr(&table->sHeader))
        return FALSE;
    if (!rhdAtomGetTableRevisionAndSize(&table->sHeader,NULL,NULL,
										&size))
        return FALSE;
# define SET_DATA_TABLE(x) {\
rhdAtomAnalyzeRomDataTable(base,data_table->x,(void *)(&(data->x)),&size); \
}
	
# define SET_DATA_TABLE_VERS(x) {\
rhdAtomAnalyzeRomDataTable(base,data_table->x,&(data->x.base),&size); \
}
	
    SET_DATA_TABLE(UtilityPipeLine);
    SET_DATA_TABLE(MultimediaCapabilityInfo);
    SET_DATA_TABLE(MultimediaConfigInfo);
    SET_DATA_TABLE(StandardVESA_Timing);
    SET_DATA_TABLE_VERS(FirmwareInfo);
    SET_DATA_TABLE(DAC_Info);
    SET_DATA_TABLE_VERS(LVDS_Info);
    SET_DATA_TABLE(TMDS_Info);
    SET_DATA_TABLE(AnalogTV_Info);
    SET_DATA_TABLE_VERS(SupportedDevicesInfo);
    SET_DATA_TABLE(GPIO_I2C_Info);
    SET_DATA_TABLE(VRAM_UsageByFirmware);
    SET_DATA_TABLE(GPIO_Pin_LUT);
    SET_DATA_TABLE(VESA_ToInternalModeLUT);
    SET_DATA_TABLE_VERS(ComponentVideoInfo);
    SET_DATA_TABLE(PowerPlayInfo);
    SET_DATA_TABLE(CompassionateData);
    SET_DATA_TABLE(SaveRestoreInfo);
    SET_DATA_TABLE(PPLL_SS_Info);
    SET_DATA_TABLE(OemInfo);
    SET_DATA_TABLE(XTMDS_Info);
    SET_DATA_TABLE(MclkSS_Info);
    SET_DATA_TABLE(Object_Header);
    SET_DATA_TABLE(IndirectIOAccess);
    SET_DATA_TABLE(MC_InitParameter);
    SET_DATA_TABLE(ASIC_VDDC_Info);
    SET_DATA_TABLE(ASIC_InternalSS_Info);
    SET_DATA_TABLE(TV_VideoMode);
    SET_DATA_TABLE_VERS(VRAM_Info);
    SET_DATA_TABLE(MemoryTrainingInfo);
    SET_DATA_TABLE_VERS(IntegratedSystemInfo);
    SET_DATA_TABLE(ASIC_ProfilingInfo);
    SET_DATA_TABLE(VoltageObjectInfo);
    SET_DATA_TABLE(PowerSourceInfo);
# undef SET_DATA_TABLE
	
    return TRUE;
}

static Bool
rhdAtomGetTables(unsigned char *base,
				 atomDataTables *atomDataPtr, unsigned char **codeTablePtr,
				 unsigned int BIOSImageSize)
{
    unsigned int  data_offset;
    unsigned int  code_offset;
    unsigned int atom_romhdr_off =  *(unsigned short*)
	(base + OFFSET_TO_POINTER_TO_ATOM_ROM_HEADER);
    ATOM_ROM_HEADER *atom_rom_hdr =
	(ATOM_ROM_HEADER *)(base + atom_romhdr_off);
	
    RHDFUNC;
	
    if (atom_romhdr_off + sizeof(ATOM_ROM_HEADER) > BIOSImageSize) {
		LOG1("%s: AtomROM header extends beyond BIOS image\n",__func__);
		return FALSE;
    }
	
    if (memcmp("ATOM",&atom_rom_hdr->uaFirmWareSignature,4)) {
        LOG1("%s: No AtomBios signature found\n", __func__);
        return FALSE;
    }
    LOG("ATOM BIOS Rom: \n");
    if (!rhdAtomAnalyzeRomHdr(base, atom_rom_hdr, &data_offset, &code_offset)) {
        LOG("RomHeader invalid\n");
        return FALSE;
    }
	
    if (data_offset + sizeof (ATOM_MASTER_DATA_TABLE) > BIOSImageSize) {
		LOG1("%s: Atom data table outside of BIOS\n", __func__);
		return FALSE;
    }
	
    if (code_offset + sizeof (ATOM_MASTER_COMMAND_TABLE) > BIOSImageSize) {
		LOG1("%s: Atom command table outside of BIOS\n", __func__);
		(*codeTablePtr) = NULL;
    } else
		(*codeTablePtr) = base + code_offset;
	
    if (!rhdAtomAnalyzeMasterDataTable(base, (ATOM_MASTER_DATA_TABLE *)
									   (base + data_offset),
									   atomDataPtr)) {
        LOG1("%s: ROM Master Table invalid\n", __func__);
        return FALSE;
    }
	
    return TRUE;
}

static AtomBiosResult
rhdAtomFirmwareInfoQuery(atomBiosHandlePtr handle, AtomBiosRequestID func, CARD32 *data)
{
    atomDataTablesPtr atomDataPtr;
    CARD8 crev, frev;
    CARD32 *val = data;
	
    RHDFUNC;
	
    atomDataPtr = handle->atomDataPtr;
	
    if (!rhdAtomGetTableRevisionAndSize(
										(ATOM_COMMON_TABLE_HEADER *)(atomDataPtr->FirmwareInfo.base),
										&crev,&frev,NULL)) {
		return ATOM_FAILED;
    }
	
    switch (crev) {
		case 1:
			switch (func) {
				case GET_DEFAULT_ENGINE_CLOCK:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo->ulDefaultEngineClock * 10;
					break;
				case GET_DEFAULT_MEMORY_CLOCK:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo->ulDefaultMemoryClock * 10;
					break;
				case GET_MAX_PIXEL_CLOCK_PLL_OUTPUT:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo->ulMaxPixelClockPLL_Output * 10;
					break;
				case GET_MIN_PIXEL_CLOCK_PLL_OUTPUT:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo->usMinPixelClockPLL_Output * 10;
				case GET_MAX_PIXEL_CLOCK_PLL_INPUT:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo->usMaxPixelClockPLL_Input * 10;
					break;
				case GET_MIN_PIXEL_CLOCK_PLL_INPUT:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo->usMinPixelClockPLL_Input * 10;
					break;
				case GET_MAX_PIXEL_CLK:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo->usMaxPixelClock * 10;
					break;
				case GET_REF_CLOCK:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo->usReferenceClock * 10;
					break;
				default:
					return ATOM_NOT_IMPLEMENTED;
			}
		case 2:
			switch (func) {
				case GET_DEFAULT_ENGINE_CLOCK:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_2->ulDefaultEngineClock * 10;
					break;
				case GET_DEFAULT_MEMORY_CLOCK:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_2->ulDefaultMemoryClock * 10;
					break;
				case GET_MAX_PIXEL_CLOCK_PLL_OUTPUT:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_2->ulMaxPixelClockPLL_Output * 10;
					break;
				case GET_MIN_PIXEL_CLOCK_PLL_OUTPUT:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_2->usMinPixelClockPLL_Output * 10;
					break;
				case GET_MAX_PIXEL_CLOCK_PLL_INPUT:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_2->usMaxPixelClockPLL_Input * 10;
					break;
				case GET_MIN_PIXEL_CLOCK_PLL_INPUT:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_2->usMinPixelClockPLL_Input * 10;
					break;
				case GET_MAX_PIXEL_CLK:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_2->usMaxPixelClock * 10;
					break;
				case GET_REF_CLOCK:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_2->usReferenceClock * 10;
					break;
				default:
					return ATOM_NOT_IMPLEMENTED;
			}
			break;
		case 3:
			switch (func) {
				case GET_DEFAULT_ENGINE_CLOCK:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_3->ulDefaultEngineClock * 10;
					break;
				case GET_DEFAULT_MEMORY_CLOCK:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_3->ulDefaultMemoryClock * 10;
					break;
				case GET_MAX_PIXEL_CLOCK_PLL_OUTPUT:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_3->ulMaxPixelClockPLL_Output * 10;
					break;
				case GET_MIN_PIXEL_CLOCK_PLL_OUTPUT:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_3->usMinPixelClockPLL_Output * 10;
					break;
				case GET_MAX_PIXEL_CLOCK_PLL_INPUT:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_3->usMaxPixelClockPLL_Input * 10;
					break;
				case GET_MIN_PIXEL_CLOCK_PLL_INPUT:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_3->usMinPixelClockPLL_Input * 10;
					break;
				case GET_MAX_PIXEL_CLK:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_3->usMaxPixelClock * 10;
					break;
				case GET_REF_CLOCK:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_3->usReferenceClock * 10;
					break;
				default:
					return ATOM_NOT_IMPLEMENTED;
			}
			break;
		case 4:
			switch (func) {
				case GET_DEFAULT_ENGINE_CLOCK:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_4->ulDefaultEngineClock * 10;
					break;
				case GET_DEFAULT_MEMORY_CLOCK:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_4->ulDefaultMemoryClock * 10;
					break;
				case GET_MAX_PIXEL_CLOCK_PLL_INPUT:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_4->usMaxPixelClockPLL_Input * 10;
					break;
				case GET_MIN_PIXEL_CLOCK_PLL_INPUT:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_4->usMinPixelClockPLL_Input * 10;
					break;
				case GET_MAX_PIXEL_CLOCK_PLL_OUTPUT:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_4->ulMaxPixelClockPLL_Output * 10;
					break;
				case GET_MIN_PIXEL_CLOCK_PLL_OUTPUT:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_4->usMinPixelClockPLL_Output * 10;
					break;
				case GET_MAX_PIXEL_CLK:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_4->usMaxPixelClock * 10;
					break;
				case GET_REF_CLOCK:
					*val = atomDataPtr->FirmwareInfo
					.FirmwareInfo_V_1_4->usReferenceClock * 10;
					break;
				default:
					return ATOM_NOT_IMPLEMENTED;
			}
			break;
		default:
			return ATOM_NOT_IMPLEMENTED;
    }
    return ATOM_SUCCESS;
}

static AtomBiosResult
rhdAtomExec (atomBiosHandlePtr handle,
			 AtomBiosRequestID unused, AtomExecPtr data)
{
    Bool ret = FALSE;
    char *msg;
    int idx = data->index;
    void *pspace = data->pspace;
    //pointer *dataSpace = data->dataSpace;
	
    RHDFUNC;
	
	/*
    if (dataSpace) {
		if (!handle->fbBase && !handle->scratchBase)
			return ATOM_FAILED;
		if (handle->fbBase) {
			if (!rhdPtr->FbBase) {
				xf86DrvMsg(handle->scrnIndex, X_ERROR, "%s: "
						   "Cannot exec AtomBIOS: framebuffer not mapped\n",
						   __func__);
				return ATOM_FAILED;
			}
			*dataSpace = (CARD8*)rhdPtr->FbBase + handle->fbBase;
		} else
			*dataSpace = (CARD8*)handle->scratchBase;
    }
	 */
    ret = ParseTableWrapper(pspace, idx, handle,
							handle->BIOSBase,
							&msg);
	LOG1("%s\n",msg);
	
    return (ret) ? ATOM_SUCCESS : ATOM_FAILED;
}

static Bool
rhdAtomASICInit(atomBiosHandlePtr handle)
{
    ASIC_INIT_PS_ALLOCATION asicInit;
	CARD32 val;
    AtomExecRec data;
	
    RHDFUNC;
	
    rhdAtomFirmwareInfoQuery(handle, GET_DEFAULT_ENGINE_CLOCK, &val);
    asicInit.sASICInitClocks.ulDefaultEngineClock = val / 10;/*in 10 Khz*/
    rhdAtomFirmwareInfoQuery(handle, GET_DEFAULT_MEMORY_CLOCK, &val);
    asicInit.sASICInitClocks.ulDefaultMemoryClock = val / 10;/*in 10 Khz*/
    data.dataSpace = NULL;
    data.index = GetIndexIntoMasterTable(COMMAND, ASIC_Init);
    data.pspace = (pointer) &asicInit;
	
    LOG("Calling ASIC Init\n");
    //atomDebugPrintPspace(handle, &data, sizeof(asicInit));
    if (rhdAtomExec(handle, ATOMBIOS_EXEC, &data) == ATOM_SUCCESS) {
		LOG("ASIC_INIT Successful\n");
		return TRUE;
    }
    LOG("ASIC_INIT Failed\n");
    return FALSE;
}

int rhdAtomInit(RHDPtr rhdPtr, unsigned char *BIOSbase, CARD32 BIOSImageSize) {
    atomDataTablesPtr atomDataPtr;
    atomBiosHandlePtr handle = NULL;
    unsigned char* codeTable;
	
    RHDFUNC;
	
    if (!(atomDataPtr = (atomDataTablesPtr) PoolAllocateResident(sizeof(atomDataTables), TRUE))) {
		LOG("Cannot allocate memory for ATOM BIOS data tabes\n");
		goto error;
    }
    if (!rhdAtomGetTables(BIOSbase, atomDataPtr, &codeTable, BIOSImageSize))
		goto error1;
    if (!(handle = (atomBiosHandlePtr) PoolAllocateResident(sizeof(atomBiosHandleRec), TRUE))) {
		LOG("Cannot allocate memory for atomBiosHandleRec\n");
		goto error1;
    }
    handle->BIOSBase = BIOSbase;
    handle->atomDataPtr = atomDataPtr;
    handle->rhdPtr = rhdPtr;
    handle->BIOSImageSize = BIOSImageSize;
    handle->codeTable = codeTable;
    handle->SaveListObjects = NULL;
	
	if (!rhdAtomASICInit(handle))
		LOG1("%s: AsicInit failed. Won't be able to obtain in VRAM FB scratch space\n",__func__);
	
    return 1;
	
error1:
    IOFree(atomDataPtr, sizeof(atomDataTables));
error:
    return 0;
}

/* all below functions are needed by ATOM BIOS PARSER */
/* these Cali functions are their virtual functions that need to be implemented by the driver */
 
#define RHDRegRead(ptr, offset) _RHDRegRead((ptr)->rhdPtr, (offset))
#define RHDRegWrite(ptr, offset, value) _RHDRegWrite((ptr)->rhdPtr, (offset), (value))
#define RHDRegMask(ptr, offset, value, mask) _RHDRegMask((ptr)->rhdPtr, (offset), (value), (mask))
#define RHDReadMC(ptr,addr) _RHDReadMC((ptr)->rhdPtr,(addr))
#define RHDWriteMC(ptr,addr,value) _RHDWriteMC((ptr)->rhdPtr,(addr),(value))
#define RHDReadPLL(ptr, off) _RHDReadPLL((ptr)->rhdPtr,(off))
#define RHDWritePLL(ptr, off, value) _RHDWritePLL((ptr)->rhdPtr,(off),(value))

/*
 *
 */
CARD32
_RHDRegRead(RHDPtr rhdPtr, CARD16 offset)
{
    return *(volatile CARD32 *)((CARD8 *) (rhdPtr->IObase) + offset);
}

/*
 *
 */
void
_RHDRegWrite(RHDPtr rhdPtr, CARD16 offset, CARD32 value)
{
    *(volatile CARD32 *)((CARD8 *) (rhdPtr->IObase) + offset) = value;
}

/*
 * This one might seem clueless, but it is an actual lifesaver.
 */
void
_RHDRegMask(RHDPtr rhdPtr, CARD16 offset, CARD32 value, CARD32 mask)
{
    CARD32 tmp;
	
    tmp = _RHDRegRead(rhdPtr, offset);
    tmp &= ~mask;
    tmp |= (value & mask);
    _RHDRegWrite(rhdPtr, offset, tmp);
}

/* The following two are R5XX only. R6XX doesn't require these */
CARD32
_RHDReadMC(RHDPtr rhdPtr, CARD32 addr)
{
    CARD32 ret;
	
    if (rhdPtr->ChipSet < RHD_RS600) {
		_RHDRegWrite(rhdPtr, MC_IND_INDEX, addr);
		ret = _RHDRegRead(rhdPtr, MC_IND_DATA);
    } else if (rhdPtr->ChipSet == RHD_RS600) {
		_RHDRegWrite(rhdPtr, RS60_MC_NB_MC_INDEX, addr);
		ret = _RHDRegRead(rhdPtr, RS60_MC_NB_MC_DATA);
    } else if (rhdPtr->ChipSet == RHD_RS690 || rhdPtr->ChipSet == RHD_RS740) {
		ExpMgrConfigWriteLong(&rhdPtr->device, (LogicalAddress) RS69_MC_INDEX, addr & ~RS69_MC_IND_WR_EN);
		ExpMgrConfigReadLong(&rhdPtr->device, (LogicalAddress) RS69_MC_DATA, &ret);
    } else {
		ExpMgrConfigWriteLong(&rhdPtr->device, (LogicalAddress) RS78_NB_MC_IND_INDEX, (addr & ~RS78_MC_IND_WR_EN));
		ExpMgrConfigReadLong(&rhdPtr->device, (LogicalAddress) RS78_NB_MC_IND_DATA, &ret);
    }
	
    LOG3("%s(0x%08X) = 0x%08X\n",__func__,(unsigned int)addr, (unsigned int)ret);
    return ret;
}

void
_RHDWriteMC(RHDPtr rhdPtr, CARD32 addr, CARD32 data)
{
	
    LOG3("%s(0x%08X, 0x%08X)\n",__func__,(unsigned int)addr, (unsigned int)data);
	
    if (rhdPtr->ChipSet < RHD_RS600) {
		_RHDRegWrite(rhdPtr, MC_IND_INDEX, addr | MC_IND_WR_EN);
		_RHDRegWrite(rhdPtr, MC_IND_DATA, data);
    } else if (rhdPtr->ChipSet == RHD_RS600) {
		_RHDRegWrite(rhdPtr, RS60_MC_NB_MC_INDEX, addr | RS60_NB_MC_IND_WR_EN);
		_RHDRegWrite(rhdPtr, RS60_MC_NB_MC_DATA, data);
    } else if (rhdPtr->ChipSet == RHD_RS690 || rhdPtr->ChipSet == RHD_RS740) {
		ExpMgrConfigWriteLong(&rhdPtr->device, (LogicalAddress) RS69_MC_INDEX, addr | RS69_MC_IND_WR_EN);
		ExpMgrConfigWriteLong(&rhdPtr->device, (LogicalAddress) RS69_MC_DATA, data);
    } else  {
		ExpMgrConfigWriteLong(&rhdPtr->device, (LogicalAddress) RS78_NB_MC_IND_INDEX, addr | RS78_MC_IND_WR_EN);
		ExpMgrConfigWriteLong(&rhdPtr->device, (LogicalAddress) RS78_NB_MC_IND_DATA, data);
    }
}

CARD32
_RHDReadPLL(RHDPtr rhdPtr, CARD16 offset)
{
    _RHDRegWrite(rhdPtr, CLOCK_CNTL_INDEX, (offset & PLL_ADDR));
    return _RHDRegRead(rhdPtr, CLOCK_CNTL_DATA);
}

void
_RHDWritePLL(RHDPtr rhdPtr, CARD16 offset, CARD32 data)
{
    _RHDRegWrite(rhdPtr, CLOCK_CNTL_INDEX, (offset & PLL_ADDR) | PLL_WR_EN);
    _RHDRegWrite(rhdPtr, CLOCK_CNTL_DATA, data);
}


#define ALLOC_CNT 25

/*
 *
 */
static void
atomSaveRegisters(atomBiosHandlePtr handle, enum atomRegisterType Type, CARD32 address)
{
    struct atomSaveListRecord *List;
    CARD32 val = 0;
    int i;
    struct atomSaveListObject *SaveListObj = handle->SaveListObjects;
	
    RHDFUNC;
	
    if (!handle->SaveList)
		return;
	
    if (!(*(handle->SaveList))) {
		if (!(*handle->SaveList = (struct atomSaveListRecord *)PoolAllocateResident(sizeof(struct atomSaveListRecord)
																	  + sizeof(struct  atomRegisterList) * (ALLOC_CNT - 1), FALSE)))
			return;
		(*(handle->SaveList))->Length = ALLOC_CNT;
		(*(handle->SaveList))->Last = 0;
    } else if ((*(handle->SaveList))->Length == (*(handle->SaveList))->Last) {
		/*
		if (!(List = (struct atomSaveListRecord *)xrealloc(*handle->SaveList,
														   sizeof(struct atomSaveListRecord)
														   + (sizeof(struct  atomRegisterList)
															  * ((*(handle->SaveList))->Length + ALLOC_CNT - 1)))))
		return; */
		if (!(List = (struct atomSaveListRecord *)PoolAllocateResident(sizeof(struct atomSaveListRecord)
														   + (sizeof(struct  atomRegisterList)
															  * ((*(handle->SaveList))->Length + ALLOC_CNT - 1)), FALSE)))
			return;
		memcpy(List, *(handle->SaveList), sizeof(struct atomSaveListRecord) + (sizeof(struct  atomRegisterList) * (*(handle->SaveList))->Length - 1));
		PoolDeallocate((LogicalAddress) *(handle->SaveList));
		*handle->SaveList = List;
		List->Length = (*(handle->SaveList))->Length + ALLOC_CNT;
    }
    List = *handle->SaveList;
	
    while (SaveListObj) {
		struct atomSaveListRecord *ListFromObj = *(SaveListObj->SaveList);
		
		if (ListFromObj) {
			for (i = 0; i < ListFromObj->Last; i++)
				if (ListFromObj->RegisterList[i].Address == address
					&& ListFromObj->RegisterList[i].Type == Type)
					return;
		}
		SaveListObj = SaveListObj->next;
    }
	
    switch (Type) {
		case atomRegisterMMIO:
			val = RHDRegRead(handle, address);
			LOG4("%s[%i]: MMIO(0x%4.4x) = 0x%4.4x\n",__func__,List->Last,address,val);
			break;
		case atomRegisterMC:
			val = RHDReadMC(handle, address | MC_IND_ALL);
			LOG4("%s[%i]: MC(0x%4.4x) = 0x%4.4x\n",__func__,List->Last,address,val);
			break;
		case atomRegisterPLL:
			val = _RHDReadPLL(handle->rhdPtr, address);
			LOG4("%s[%i]: PLL(0x%4.4x) = 0x%4.4x\n",__func__,List->Last,address,val);
			break;
		case atomRegisterPCICFG:
			ExpMgrConfigReadLong(&handle->rhdPtr->device, (LogicalAddress) address, &val);
			LOG4("%s[%i]: PCICFG(0x%4.4x) = 0x%4.4x\n",__func__,List->Last,address,val);
			break;
    }
    List->RegisterList[List->Last].Address = address;
    List->RegisterList[List->Last].Value = val;
    List->RegisterList[List->Last].Type = Type;
    List->Last++;
}

/*
 *
 */
VOID*
CailAllocateMemory(VOID *CAIL,UINT16 size)
{
    RHDFUNC;
	
    return PoolAllocateResident(size, FALSE);
}

VOID
CailReleaseMemory(VOID *CAIL, VOID *addr)
{
    RHDFUNC;
	
    PoolDeallocate(addr);
}

VOID
CailDelayMicroSeconds(VOID *CAIL, UINT32 delay)
{
    RHDFUNC;
	
    IODelay(delay);
	
    LOG1("Delay %i usec\n",delay);
}

UINT32
CailReadATIRegister(VOID* CAIL, UINT32 idx)
{
    UINT32 ret;
    RHDFUNC;
	
    ret  =  RHDRegRead(((atomBiosHandlePtr)CAIL), idx << 2);
    LOG3("%s(%x) = %x\n",__func__,idx << 2,ret);
    return ret;
}

VOID
CailWriteATIRegister(VOID *CAIL, UINT32 idx, UINT32 data)
{
    RHDFUNC;
	
    atomSaveRegisters((atomBiosHandlePtr)CAIL, atomRegisterMMIO, idx << 2);
	
    RHDRegWrite(((atomBiosHandlePtr)CAIL),idx << 2,data);
    LOG3("%s(%x,%x)\n",__func__,idx << 2,data);
}

UINT32
CailReadFBData(VOID* CAIL, UINT32 idx)
{
    UINT32 ret;
	
    RHDFUNC;
	
    if (((atomBiosHandlePtr)CAIL)->fbBase) {
		CARD8 *FBBase = (CARD8*)
	    RHDPTRI((atomBiosHandlePtr)CAIL)->FBbase;
		ret =  *((CARD32*)(FBBase + (((atomBiosHandlePtr)CAIL)->fbBase) + idx));
		LOG3("%s(%x) = %x\n",__func__,idx,ret);
    } else if (((atomBiosHandlePtr)CAIL)->scratchBase) {
		ret = *(CARD32*)((CARD8*)(((atomBiosHandlePtr)CAIL)->scratchBase) + idx);
		LOG3("%s(%x) = %x\n",__func__,idx,ret);
    } else {
		LOG1("%s: no fbbase set\n",__func__);
		return 0;
    }
    return ret;
}

VOID
CailWriteFBData(VOID *CAIL, UINT32 idx, UINT32 data)
{
    RHDFUNC;
	
    LOG3("%s(%x,%x)\n",__func__,idx,data);
    if (((atomBiosHandlePtr)CAIL)->fbBase) {
		CARD8 *FBBase = (CARD8*)
	    RHDPTRI((atomBiosHandlePtr)CAIL)->FBbase;
		*((CARD32*)(FBBase + (((atomBiosHandlePtr)CAIL)->fbBase) + idx)) = data;
    } else if (((atomBiosHandlePtr)CAIL)->scratchBase) {
		*(CARD32*)((CARD8*)(((atomBiosHandlePtr)CAIL)->scratchBase) + idx) = data;
    } else
		LOG1("%s: no fbbase set\n",__func__);
}

ULONG
CailReadMC(VOID *CAIL, ULONG Address)
{
    ULONG ret;
	
    RHDFUNC;
	
    ret = RHDReadMC(((atomBiosHandlePtr)CAIL), Address | MC_IND_ALL);
    LOG3("%s(%x) = %x\n",__func__,Address,ret);
    return ret;
}

VOID
CailWriteMC(VOID *CAIL, ULONG Address, ULONG data)
{
    RHDFUNC;
	
	
    LOG3("%s(%x,%x)\n",__func__,Address,data);
	
    atomSaveRegisters((atomBiosHandlePtr)CAIL, atomRegisterMC, Address);
	
    RHDWriteMC(((atomBiosHandlePtr)CAIL), Address | MC_IND_ALL | MC_IND_WR_EN, data);
}

VOID
CailReadPCIConfigData(VOID*CAIL, VOID* ret, UINT32 idx,UINT16 size)
{
    RegEntryID *reg = &RHDPTRI((atomBiosHandlePtr)CAIL)->device;
	
    RHDFUNC;
	
    switch (size) {
		case 8:
			ExpMgrConfigReadByte(reg,(LogicalAddress) (idx << 2), (CARD8*)ret);
			break;
		case 16:
			ExpMgrConfigReadWord(reg,(LogicalAddress) (idx << 2), (CARD16*)ret);
			break;
		case 32:
			ExpMgrConfigReadLong(reg,(LogicalAddress) (idx << 2), (CARD32*)ret);
			break;
		default:
			LOG2("%s: Unsupported size: %i\n", __func__,(int)size);
			return;
			break;
    }
    LOG3("%s(%x) = %x\n",__func__,idx,*(unsigned int*)ret);
	
}

VOID
CailWritePCIConfigData(VOID*CAIL,VOID*src,UINT32 idx,UINT16 size)
{
    RegEntryID *reg = &RHDPTRI((atomBiosHandlePtr)CAIL)->device;
	
    RHDFUNC;
	
    LOG3("%s(%x,%x)\n",__func__,idx,(*(unsigned int*)src));
	
    atomSaveRegisters((atomBiosHandlePtr)CAIL, atomRegisterPCICFG, idx << 2);
	
    switch (size) {
		case 8:
			ExpMgrConfigWriteByte(reg,(LogicalAddress) (idx << 2),*(CARD8*)src);
			break;
		case 16:
			ExpMgrConfigWriteWord(reg,(LogicalAddress) (idx << 2),*(CARD16*)src);
			break;
		case 32:
			ExpMgrConfigWriteLong(reg,(LogicalAddress) (idx << 2),*(CARD32*)src);
			break;
		default:
			LOG2("%s: Unsupported size: %i\n",__func__,(int)size);
			break;
    }
}

ULONG
CailReadPLL(VOID *CAIL, ULONG Address)
{
    ULONG ret;
	
    RHDFUNC;
	
    ret = _RHDReadPLL(((atomBiosHandlePtr)CAIL)->rhdPtr, Address);
    LOG3("%s(%x) = %x\n",__func__,Address,ret);
    return ret;
}

VOID
CailWritePLL(VOID *CAIL, ULONG Address,ULONG Data)
{
    RHDFUNC;
	
    LOG3("%s(%x,%x)\n",__func__,Address,Data);
    atomSaveRegisters((atomBiosHandlePtr)CAIL, atomRegisterPLL, Address);
    _RHDWritePLL(((atomBiosHandlePtr)CAIL)->rhdPtr, Address, Data);
}

