/*
 *  rhd_init.h
 *  ATY_Init
 *
 *  Created by Dong Luo on 3/20/09.
 *  Copyright 2009 Boston University. All rights reserved.
 *
 */

#ifndef _RHD_INIT_H
# define _RHD_INIT_H

typedef char Bool;
#if __LP64__
typedef unsigned int       CARD32;
#else
typedef unsigned long      CARD32;
#endif
typedef unsigned short CARD16;
typedef unsigned char CARD8;
typedef char * pointer;
typedef struct _atomBiosHandle *atomBiosHandlePtr;

#define ATOM_BIOS
#define ATOM_BIOS_PARSER

#define RHD_ATOMBIOS_ON 1
#define RHD_ATOMBIOS_OFF 2
#define RHD_ATOMBIOS_FORCE 4
#define RHD_ATOMBIOS_CRTC 0
#define RHD_ATOMBIOS_PLL 4
#define RHD_ATOMBIOS_OUTPUT 8

typedef enum _AtomBiosRequestID {
    ATOMBIOS_INIT,
    ATOMBIOS_TEARDOWN,
    ATOMBIOS_EXEC,
    ATOMBIOS_ALLOCATE_FB_SCRATCH,
    ATOMBIOS_GET_CONNECTORS,
    ATOMBIOS_GET_OUTPUT_DEVICE_LIST,
    ATOMBIOS_GET_PANEL_MODE,
    ATOMBIOS_GET_PANEL_EDID,
    ATOMBIOS_GET_CODE_DATA_TABLE,
    GET_DEFAULT_ENGINE_CLOCK,
    GET_DEFAULT_MEMORY_CLOCK,
    GET_MAX_PIXEL_CLOCK_PLL_OUTPUT,
    GET_MIN_PIXEL_CLOCK_PLL_OUTPUT,
    GET_MAX_PIXEL_CLOCK_PLL_INPUT,
    GET_MIN_PIXEL_CLOCK_PLL_INPUT,
    GET_MAX_PIXEL_CLK,
    GET_REF_CLOCK,
    GET_FW_FB_START,
    GET_FW_FB_SIZE,
    ATOM_TMDS_MAX_FREQUENCY,
    ATOM_TMDS_PLL_CHARGE_PUMP,
    ATOM_TMDS_PLL_DUTY_CYCLE,
    ATOM_TMDS_PLL_VCO_GAIN,
    ATOM_TMDS_PLL_VOLTAGE_SWING,
    ATOM_LVDS_SUPPORTED_REFRESH_RATE,
    ATOM_LVDS_OFF_DELAY,
    ATOM_LVDS_SEQ_DIG_ONTO_DE,
    ATOM_LVDS_SEQ_DE_TO_BL,
    ATOM_LVDS_SPATIAL_DITHER,
    ATOM_LVDS_TEMPORAL_DITHER,
    ATOM_LVDS_DUALLINK,
    ATOM_LVDS_24BIT,
    ATOM_LVDS_GREYLVL,
    ATOM_LVDS_FPDI,
    ATOM_GPIO_QUERIES,
    ATOM_GPIO_I2C_CLK_MASK,
    ATOM_GPIO_I2C_CLK_MASK_SHIFT,
    ATOM_GPIO_I2C_DATA_MASK,
    ATOM_GPIO_I2C_DATA_MASK_SHIFT,
    ATOM_DAC1_BG_ADJ,
    ATOM_DAC1_DAC_ADJ,
    ATOM_DAC1_FORCE,
    ATOM_DAC2_CRTC2_BG_ADJ,
    ATOM_DAC2_NTSC_BG_ADJ,
    ATOM_DAC2_PAL_BG_ADJ,
    ATOM_DAC2_CV_BG_ADJ,
    ATOM_DAC2_CRTC2_DAC_ADJ,
    ATOM_DAC2_NTSC_DAC_ADJ,
    ATOM_DAC2_PAL_DAC_ADJ,
    ATOM_DAC2_CV_DAC_ADJ,
    ATOM_DAC2_CRTC2_FORCE,
    ATOM_DAC2_CRTC2_MUX_REG_IND,
    ATOM_DAC2_CRTC2_MUX_REG_INFO,
    ATOM_ANALOG_TV_MODE,
    ATOM_ANALOG_TV_DEFAULT_MODE,
    ATOM_ANALOG_TV_SUPPORTED_MODES,
    ATOM_GET_CONDITIONAL_GOLDEN_SETTINGS,
    ATOM_GET_PCIENB_CFG_REG7,
    ATOM_GET_CAPABILITY_FLAG,
    ATOM_GET_PCIE_LANES,
    ATOM_SET_REGISTER_LIST_LOCATION,
    ATOM_RESTORE_REGISTERS,
    FUNC_END
} AtomBiosRequestID;

typedef enum _AtomBiosResult {
    ATOM_SUCCESS,
    ATOM_FAILED,
    ATOM_NOT_IMPLEMENTED
} AtomBiosResult;

typedef struct AtomExec {
    int index;
    pointer pspace;
    pointer *dataSpace;
} AtomExecRec, *AtomExecPtr;

typedef struct AtomFb {
    unsigned int start;
    unsigned int size;
} AtomFbRec, *AtomFbPtr;

struct AtomDacCodeTableData
{
    CARD8 DAC1PALWhiteFine;
    CARD8 DAC1PALBandGap;
    CARD8 DAC1NTSCWhiteFine;
    CARD8 DAC1NTSCBandGap;
    CARD8 DAC1VGAWhiteFine;
    CARD8 DAC1VGABandGap;
    CARD8 DAC1CVWhiteFine;
    CARD8 DAC1CVBandGap;
    CARD8 DAC2PALWhiteFine;
    CARD8 DAC2PALBandGap;
    CARD8 DAC2NTSCWhiteFine;
    CARD8 DAC2NTSCBandGap;
    CARD8 DAC2VGAWhiteFine;
    CARD8 DAC2VGABandGap;
    CARD8 DAC2CVWhiteFine;
    CARD8 DAC2CVBandGap;
};

typedef enum AtomTVMode {
    ATOM_TVMODE_NTSC = 1 << 0,
    ATOM_TVMODE_NTSCJ = 1 << 1,
    ATOM_TVMODE_PAL = 1 << 2,
    ATOM_TVMODE_PALM = 1 << 3,
    ATOM_TVMODE_PALCN = 1 << 4,
    ATOM_TVMODE_PALN = 1 << 5,
    ATOM_TVMODE_PAL60 = 1 << 6,
    ATOM_TVMODE_SECAM = 1 << 7,
    ATOM_TVMODE_CV = 1 << 8
} AtomTVMode;

enum atomPCIELanes {
    atomPCIELaneNONE,
    atomPCIELane0_3,
    atomPCIELane0_7,
    atomPCIELane4_7,
    atomPCIELane8_11,
    atomPCIELane8_15,
    atomPCIELane12_15
};

enum atomDevice {
    atomNone, /* 0 */
    atomCRT1, /* 1 */
    atomLCD1, /* 2 */
    atomTV1,  /* 3 */
    atomDFP1, /* 4 */
    atomCRT2, /* 5 */
    atomLCD2, /* 6 */
    atomTV2,  /* 7 */
    atomDFP2, /* 8 */
    atomCV,   /* 9 */
    atomDFP3, /* a */
    atomDFP4, /* b */
    atomDFP5  /* c */
};

typedef struct AtomGoldenSettings
	{
		unsigned char *BIOSPtr;
		unsigned char *End;
		unsigned int value;
		
	} AtomGoldenSettings;

enum atomCrtc {
    atomCrtc1,
    atomCrtc2
};

enum atomCrtcAction {
    atomCrtcEnable,
    atomCrtcDisable
};

enum atomOutputLinks {
    atomSingleLink,
    atomDualLink
};

enum atomTransmitter {
    atomTransmitterLVTMA,
    atomTransmitterUNIPHY,
    atomTransmitterUNIPHY1,
    atomTransmitterUNIPHY2,
    atomTransmitterPCIEPHY,
    atomTransmitterDIG1,
    atomTransmitterDIG2
};

enum atomTransmitterAction {
    atomTransDisable,
    atomTransEnable,
    atomTransEnableOutput,
    atomTransDisableOutput,
    atomTransInit,
    atomTransLcdBlOff,
    atomTransLcdBlOn,
    atomTransLcdBlBrightness,
    atomTransSetup
};

enum atomEncoder {
    atomEncoderNone,
    atomEncoderDACA,
    atomEncoderDACB,
    atomEncoderTV,
    atomEncoderTMDS1,  /* TMDSA */
    atomEncoderTMDS2,  /* LVTMA */
    atomEncoderLVDS,   /* LVTMA (Panel) */
    atomEncoderDVO,
    atomEncoderDIG1,
    atomEncoderDIG2,
    atomEncoderExternal
};

enum atomEncoderMode {
    atomNoEncoder,
    atomDVI,
    atomDP,
    atomLVDS,
    atomHDMI,
    atomSDVO,
    atomTVComposite,
    atomTVSVideo,
    atomTVComponent,
    atomCRT
};

enum atomEncoderAction {
    atomEncoderOff,
    atomEncoderOn
};

enum atomOutput {
    atomDVOOutput,
    atomLCDOutput,
    atomCVOutput,
    atomTVOutput,
    atomLVTMAOutput,
    atomTMDSAOutput,
    atomDAC1Output,
    atomDAC2Output
};

enum atomOutputType {
    atomOutputNone,
    atomOutputDacA,
    atomOutputDacB,
    atomOutputTmdsa,
    atomOutputLvtma,
    atomOutputDvo,
    atomOutputKldskpLvtma,
    atomOutputUniphyA,
    atomOutputUniphyB,
    atomOutputUniphyC,
    atomOutputUniphyD,
    atomOutputUniphyE,
    atomOutputUniphyF
};

enum atomOutputAction {
    atomOutputEnable,
    atomOutputDisable,
    atomOutputLcdOn,
    atomOutputLcdOff,
    atomOutputLcdBrightnessControl,
    atomOutputLcdSelftestStart,
    atomOutputLcdSelftestStop,
    atomOutputEncoderInit
};

enum atomDAC {
    atomDACA,
    atomDACB,
    atomDACExt
};

enum atomTransmitterLink {
    atomTransLinkA,
    atomTransLinkAB,
    atomTransLinkB,
    atomTransLinkBA
};

enum atomDACStandard {
    atomDAC_VGA,
    atomDAC_CV,
    atomDAC_NTSC,
    atomDAC_PAL
};

enum atomDVORate {
    atomDVO_RateSDR,
    atomDVO_RateDDR
};

enum atomDVOOutput {
    atomDVO_OutputLow12Bit,
    atomDVO_OutputHigh12Bit,
    atomDVO_Output24Bit
};

enum atomScaler {
    atomScaler1,
    atomScaler2
};

enum atomScaleMode {
    atomScaleDisable,
    atomScaleCenter,
    atomScaleExpand,
    atomScaleMulttabExpand
};

enum atomPxclk {
    atomPclk1,
    atomPclk2
};

struct atomCodeTableVersion
{
    CARD8 cref;
    CARD8 fref;
};

enum atomTemporalGreyLevels {
    atomTemporalDither0,
    atomTemporalDither4,
    atomTemporalDither2
};

struct atomTransmitterConfig
{
    int PixelClock;
    enum atomEncoder Encoder;
    enum atomPCIELanes Lanes;
    enum atomEncoderMode Mode;
    enum atomTransmitterLink Link;
    enum atomOutputLinks LinkCnt;
    Bool Coherent;
};

#endif /*  RHD_INIT_H_ */
