/*
 *  rhd_rec.h
 *  ATY_Init
 *
 *  Created by Dong Luo on 3/20/09.
 *  Copyright 2009 Boston University. All rights reserved.
 *
 */
#ifndef _RHD_REC_H
#define _RHD_REC_H

enum RHD_CHIPSETS {
    RHD_UNKNOWN = 0,
    /* R500 */
    RHD_RV505,
    RHD_RV515,
    RHD_RV516,
    RHD_R520,
    RHD_RV530,
    RHD_RV535,
    RHD_RV550,
    RHD_RV560,
    RHD_RV570,
    RHD_R580,
    /* R500 Mobility */
    RHD_M52,
    RHD_M54,
    RHD_M56,
    RHD_M58,
    RHD_M62,
    RHD_M64,
    RHD_M66,
    RHD_M68,
    RHD_M71,
    /* R500 integrated */
    RHD_RS600,
    RHD_RS690,
    RHD_RS740,
    /* R600 */
    RHD_R600,
    RHD_RV610,
    RHD_RV630,
    /* R600 Mobility */
    RHD_M72,
    RHD_M74,
    RHD_M76,
    /* RV670 came into existence after RV6x0 and M7x */
    RHD_RV670,
    RHD_M88,
    RHD_R680,
    RHD_RV620,
    RHD_M82,
    RHD_RV635,
    RHD_M86,
    RHD_RS780,
    RHD_RV770,
    RHD_R700,
    RHD_M98,
    RHD_RV730,
    RHD_M96,
    RHD_RV710,
    RHD_CHIP_END
};

typedef struct {
	int					PCIid;
	enum RHD_CHIPSETS	numChipset;
} PciChipSets;

const PciChipSets RHDPCIchipsets[] = {
    {  0x7100, RHD_R520  }, /* Radeon X1800 */
    {  0x7101, RHD_M58   }, /* Mobility Radeon X1800 XT */
    {  0x7102, RHD_M58   }, /* Mobility Radeon X1800 */
    {  0x7103, RHD_M58   }, /* Mobility FireGL V7200 */
    {  0x7104, RHD_R520  }, /* FireGL V7200 */
    {  0x7105, RHD_R520  }, /* FireGL V5300 */
    {  0x7106, RHD_M58   }, /* Mobility FireGL V7100 */
    {  0x7108, RHD_R520  }, /* Radeon X1800 */
    {  0x7109, RHD_R520  }, /* Radeon X1800 */
    {  0x710A, RHD_R520  }, /* Radeon X1800 */
    {  0x710B, RHD_R520  }, /* Radeon X1800 */
    {  0x710C, RHD_R520  }, /* Radeon X1800 */
    {  0x710E, RHD_R520  }, /* FireGL V7300 */
    {  0x710F, RHD_R520  }, /* FireGL V7350 */
    {  0x7140, RHD_RV515 }, /* Radeon X1600/X1550 */
    {  0x7141, RHD_RV505 }, /* RV505 */
    {  0x7142, RHD_RV515 }, /* Radeon X1300/X1550 */
    {  0x7143, RHD_RV505 }, /* Radeon X1550 */
    {  0x7144, RHD_M54   }, /* M54-GL */
    {  0x7145, RHD_M54   }, /* Mobility Radeon X1400 */
    {  0x7146, RHD_RV515 }, /* Radeon X1300/X1550 */
    {  0x7147, RHD_RV505 }, /* Radeon X1550 64-bit */
    {  0x7149, RHD_M52   }, /* Mobility Radeon X1300 */
    {  0x714A, RHD_M52   }, /* Mobility Radeon X1300 */
    {  0x714B, RHD_M52   }, /* Mobility Radeon X1300 */
    {  0x714C, RHD_M52   }, /* Mobility Radeon X1300 */
    {  0x714D, RHD_RV515 }, /* Radeon X1300 */
    {  0x714E, RHD_RV515 }, /* Radeon X1300 */
    {  0x714F, RHD_RV505 }, /* RV505 */
    {  0x7151, RHD_RV505 }, /* RV505 */
    {  0x7152, RHD_RV515 }, /* FireGL V3300 */
    {  0x7153, RHD_RV515 }, /* FireGL V3350 */
    {  0x715E, RHD_RV515 }, /* Radeon X1300 */
    {  0x715F, RHD_RV505 }, /* Radeon X1550 64-bit */
    {  0x7180, RHD_RV516 }, /* Radeon X1300/X1550 */
    {  0x7181, RHD_RV516 }, /* Radeon X1600 */
    {  0x7183, RHD_RV516 }, /* Radeon X1300/X1550 */
    {  0x7186, RHD_M64   }, /* Mobility Radeon X1450 */
    {  0x7187, RHD_RV516 }, /* Radeon X1300/X1550 */
    {  0x7188, RHD_M64   }, /* Mobility Radeon X2300 */
    {  0x718A, RHD_M64   }, /* Mobility Radeon X2300 */
    {  0x718B, RHD_M62   }, /* Mobility Radeon X1350 */
    {  0x718C, RHD_M62   }, /* Mobility Radeon X1350 */
    {  0x718D, RHD_M64   }, /* Mobility Radeon X1450 */
    {  0x718F, RHD_RV516 }, /* Radeon X1300 */
    {  0x7193, RHD_RV516 }, /* Radeon X1550 */
    {  0x7196, RHD_M62   }, /* Mobility Radeon X1350 */
    {  0x719B, RHD_RV516 }, /* FireMV 2250 */
    {  0x719F, RHD_RV516 }, /* Radeon X1550 64-bit */
    {  0x71C0, RHD_RV530 }, /* Radeon X1600 */
    {  0x71C1, RHD_RV535 }, /* Radeon X1650 */
    {  0x71C2, RHD_RV530 }, /* Radeon X1600 */
    {  0x71C3, RHD_RV535 }, /* Radeon X1600 */
    {  0x71C4, RHD_M56   }, /* Mobility FireGL V5200 */
    {  0x71C5, RHD_M56   }, /* Mobility Radeon X1600 */
    {  0x71C6, RHD_RV530 }, /* Radeon X1650 */
    {  0x71C7, RHD_RV535 }, /* Radeon X1650 */
    {  0x71CD, RHD_RV530 }, /* Radeon X1600 */
    {  0x71CE, RHD_RV530 }, /* Radeon X1300 XT/X1600 Pro */
    {  0x71D2, RHD_RV530 }, /* FireGL V3400 */
    {  0x71D4, RHD_M66   }, /* Mobility FireGL V5250 */
    {  0x71D5, RHD_M66   }, /* Mobility Radeon X1700 */
    {  0x71D6, RHD_M66   }, /* Mobility Radeon X1700 XT */
    {  0x71DA, RHD_RV530 }, /* FireGL V5200 */
    {  0x71DE, RHD_M66   }, /* Mobility Radeon X1700 */
    {  0x7200, RHD_RV550 }, /*  Radeon X2300HD  */
    {  0x7210, RHD_M71   }, /* Mobility Radeon HD 2300 */
    {  0x7211, RHD_M71   }, /* Mobility Radeon HD 2300 */
    {  0x7240, RHD_R580  }, /* Radeon X1950 */
    {  0x7243, RHD_R580  }, /* Radeon X1900 */
    {  0x7244, RHD_R580  }, /* Radeon X1950 */
    {  0x7245, RHD_R580  }, /* Radeon X1900 */
    {  0x7246, RHD_R580  }, /* Radeon X1900 */
    {  0x7247, RHD_R580  }, /* Radeon X1900 */
    {  0x7248, RHD_R580  }, /* Radeon X1900 */
    {  0x7249, RHD_R580  }, /* Radeon X1900 */
    {  0x724A, RHD_R580  }, /* Radeon X1900 */
    {  0x724B, RHD_R580  }, /* Radeon X1900 */
    {  0x724C, RHD_R580  }, /* Radeon X1900 */
    {  0x724D, RHD_R580  }, /* Radeon X1900 */
    {  0x724E, RHD_R580  }, /* AMD Stream Processor */
    {  0x724F, RHD_R580  }, /* Radeon X1900 */
    {  0x7280, RHD_RV570 }, /* Radeon X1950 */
    {  0x7281, RHD_RV560 }, /* RV560 */
    {  0x7283, RHD_RV560 }, /* RV560 */
    {  0x7284, RHD_M68   }, /* Mobility Radeon X1900 */
    {  0x7287, RHD_RV560 }, /* RV560 */
    {  0x7288, RHD_RV570 }, /* Radeon X1950 GT */
    {  0x7289, RHD_RV570 }, /* RV570 */
    {  0x728B, RHD_RV570 }, /* RV570 */
    {  0x728C, RHD_RV570 }, /* ATI FireGL V7400  */
    {  0x7290, RHD_RV560 }, /* RV560 */
    {  0x7291, RHD_RV560 }, /* Radeon X1650 */
    {  0x7293, RHD_RV560 }, /* Radeon X1650 */
    {  0x7297, RHD_RV560 }, /* RV560 */
    {  0x791E, RHD_RS690 }, /* Radeon X1200 */
    {  0x791F, RHD_RS690 }, /* Radeon X1200 */
    {  0x793F, RHD_RS600 }, /* Radeon Xpress 1200 */
    {  0x7941, RHD_RS600 }, /* Radeon Xpress 1200 */
    {  0x7942, RHD_RS600 }, /* Radeon Xpress 1200 (M) */
    {  0x796C, RHD_RS740 }, /* RS740 */
    {  0x796D, RHD_RS740 }, /* RS740M */
    {  0x796E, RHD_RS740 }, /* ATI Radeon 2100 RS740 */
    {  0x796F, RHD_RS740 }, /* RS740M */
    {  0x9400, RHD_R600  }, /* Radeon HD 2900 XT */
    {  0x9401, RHD_R600  }, /* Radeon HD 2900 XT */
    {  0x9402, RHD_R600  }, /* Radeon HD 2900 XT */
    {  0x9403, RHD_R600  }, /* Radeon HD 2900 Pro */
    {  0x9405, RHD_R600  }, /* Radeon HD 2900 GT */
    {  0x940A, RHD_R600  }, /* FireGL V8650 */
    {  0x940B, RHD_R600  }, /* FireGL V8600 */
    {  0x940F, RHD_R600  }, /* FireGL V7600 */
    {  0x9440, RHD_RV770 }, /* ATI Radeon 4800 Series  */
    {  0x9441, RHD_RV770 }, /* ATI Radeon 4870 X2  */
    {  0x9442, RHD_RV770 }, /* ATI Radeon 4800 Series  */
//        {  0x9443, RHD_R700  }, /* ATI Radeon 4800 Series  */
    {  0x9444, RHD_RV770 }, /* Everest ATI FirePro Graphics Accelerator  */
    {  0x9446, RHD_RV770 }, /* K2 ATI FirePro Graphics Accelerator  */
//        {  0x9447, RHD_R700 }, /* K2 ATI FirePro Graphics Accelerator  */
    {  0x944A, RHD_M98 }, /* M98  */
    {  0x944B, RHD_M98 }, /* M98  */
    {  0x944E, RHD_RV770 }, /* RV770  */
//        {  0x944F, RHD_R700 }, /* R700  */
    {  0x9456, RHD_RV770 }, /* Denali ATI FirePro Graphics Accelerator  */
    {  0x945A, RHD_M98 }, /* M98 */
    {  0x945B, RHD_M98 }, /* M98 */
    {  0x946A, RHD_M98 }, /* M98 */
    {  0x946B, RHD_M98 }, /* M98 */
    {  0x947A, RHD_M98 }, /* M98 */
    {  0x947B, RHD_M98 }, /* M96 */
    {  0x9480, RHD_M96 }, /* M98 */
    {  0x9487, RHD_RV730 }, /* RV730 */
    {  0x9488, RHD_M96 }, /* M96 */
    {  0x9489, RHD_M96 }, /* M96M GL */
    {  0x948F, RHD_RV730 }, /* RV730 */
    {  0x9487, RHD_RV730 }, /* RV730 */
    {  0x9490, RHD_RV730 }, /* RV730 */
    {  0x9498, RHD_RV730 }, /* RV730 */
    {  0x949E, RHD_RV730 }, /* RV730 */
    {  0x949F, RHD_RV730 }, /* RV730 */
    {  0x94C0, RHD_RV610 }, /* RV610 */
    {  0x94C1, RHD_RV610 }, /* Radeon HD 2400 XT */
    {  0x94C3, RHD_RV610 }, /* Radeon HD 2400 Pro */
    {  0x94C4, RHD_RV610 }, /* ATI Radeon HD 2400 PRO AGP */
    {  0x94C5, RHD_RV610 }, /* FireGL V4000 */
    {  0x94C6, RHD_RV610 }, /* RV610 */
    {  0x94C7, RHD_RV610 }, /* ATI Radeon HD 2350 */
    {  0x94C8, RHD_M74   }, /* Mobility Radeon HD 2400 XT */
    {  0x94C9, RHD_M72   }, /* Mobility Radeon HD 2400 */
    {  0x94CB, RHD_M72   }, /* ATI RADEON E2400 */
    {  0x94CC, RHD_RV610 }, /* ATI Radeon HD 2400 */
    {  0x9500, RHD_RV670 }, /* RV670 */
    {  0x9501, RHD_RV670 }, /* ATI Radeon HD3870 */
    {  0x9504, RHD_M88   }, /*  ATI Mobility Radeon HD 3850 */
    {  0x9505, RHD_RV670 }, /* ATI Radeon HD3850 */
    {  0x9506, RHD_M88   }, /*  ATI Mobility Radeon HD 3850 X2 */
    {  0x9507, RHD_RV670 }, /* RV670 */
    {  0x9508, RHD_M88   }, /*  ATI Mobility Radeon HD 3870 */
    {  0x9509, RHD_M88   }, /*  ATI Mobility Radeon HD 3870 X2 */
    {  0x950F, RHD_R680  }, /* ATI Radeon HD3870 X2 */
    {  0x9511, RHD_RV670 }, /* ATI FireGL V7700 */
    {  0x9515, RHD_RV670 }, /* ATI Radeon HD 3850 AGP */
    {  0x9517, RHD_RV670 }, /* ATI Radeon HD 3960 */
    {  0x9519, RHD_RV670 }, /* FireStream 9170 */
    {  0x9540, RHD_RV710 }, /*   */
    {  0x9541, RHD_RV710 }, /*   */
    {  0x9542, RHD_RV710 }, /*   */
    {  0x954E, RHD_RV710 }, /*   */
    {  0x954f, RHD_RV710 }, /*   */
    {  0x9580, RHD_RV630 }, /* RV630 */
    {  0x9581, RHD_M76   }, /* Mobility Radeon HD 2600 */
    {  0x9583, RHD_M76   }, /* Mobility Radeon HD 2600 XT */
    {  0x9586, RHD_RV630 }, /* ATI Radeon HD 2600 XT AGP */
    {  0x9587, RHD_RV630 }, /* ATI Radeon HD 2600 Pro AGP */
    {  0x9588, RHD_RV630 }, /* Radeon HD 2600 XT */
    {  0x9589, RHD_RV630 }, /* Radeon HD 2600 Pro */
    {  0x958A, RHD_RV630 }, /* Gemini RV630 */
    {  0x958B, RHD_M76   }, /* Gemini ATI Mobility Radeon HD 2600 XT */
    {  0x958C, RHD_RV630 }, /* FireGL V5600 */
    {  0x958D, RHD_RV630 }, /* FireGL V3600 */
    {  0x958E, RHD_RV630 }, /* ATI Radeon HD 2600 LE */
    {  0x958F, RHD_M76   }, /* ATI Mobility FireGL Graphics Processor */
    {  0x9590, RHD_RV635 }, /* ATI Radeon HD 3600 Series */
    {  0x9591, RHD_M86   }, /* Mobility Radeon HD 3650 */
    {  0x9593, RHD_M86   }, /* Mobility Radeon HD 3670 */
    {  0x9595, RHD_M86   }, /* Mobility FireGL V5700 */
    {  0x9596, RHD_RV635 }, /* ATI Radeon HD 3650 AGP */
    {  0x9597, RHD_RV635 }, /* ATI Radeon HD 3600 Series */
    {  0x9598, RHD_RV635 }, /* ATI Radeon HD 3670 */
    {  0x9599, RHD_RV635 }, /* ATI Radeon HD 3600 Series */
    {  0x959B, RHD_M86   }, /* Mobility FireGL Graphics Processor */
    {  0x95C0, RHD_RV620 }, /* ATI Radeon HD 3470 */
    {  0x95C2, RHD_M82   }, /* ATI Mobility Radeon HD 3430 (M82) */
    {  0x95C4, RHD_M82   }, /* Mobility Radeon HD 3400 Series (M82)  */
    {  0x95C5, RHD_RV620 }, /* ATI Radeon HD 3450 */
    {  0x95C6, RHD_RV620 }, /* ATI Radeon HD 3450 */
    {  0x95C7, RHD_RV620 }, /* ATI Radeon HD 3430 */
    {  0x95CC, RHD_RV620 }, /* Fire PRO Professional Graphics ASIC  */
    {  0x95CD, RHD_RV620 }, /* ATI FireMV 2450  */
    {  0x95CE, RHD_RV620 }, /* ATI FireMV 2260  */
    {  0x95CF, RHD_RV620 }, /* ATI FireMV 2260  */
    {  0x9610, RHD_RS780 }, /* ATI Radeon HD 3200 Graphics */
    {  0x9611, RHD_RS780 }, /* ATI Radeon 3100 Graphics */
    {  0x9612, RHD_RS780 }, /*  ATI Radeon HD 3200 Graphics  */
    {  0x9613, RHD_RS780 }, /* ATI Radeon 3100 Graphics   */
    {  0x9614, RHD_RS780 }, /* ATI Radeon HD 3300 Graphics  */
	{  0, RHD_UNKNOWN  }
};

#ifndef __IONDRVLIBRARIES__

struct RegEntryID
{
    void * opaque[4];
};
typedef struct RegEntryID RegEntryID;
typedef RegEntryID *                    RegEntryIDPtr;

#endif

#define MAKE_REG_ENTRY(regEntryID,obj) 				\
(regEntryID)->opaque[ 0 ] = (void *) obj;			\
(regEntryID)->opaque[ 1 ] = (void *) ~(uintptr_t)obj;		\
(regEntryID)->opaque[ 2 ] = (void *) 0x53696d65;		\
(regEntryID)->opaque[ 3 ] = (void *) 0x52756c7a;

typedef struct RHDRec {
	RegEntryID			device;
	int					chipType;
    enum RHD_CHIPSETS   ChipSet;
	void				*IObase;
	void				*FBbase;
} RHDRec, *RHDPtr;

#define RHDPTR(p) ((p)->rhdPtr)
#define RHDPTRI(p) ((p)->rhdPtr)

#endif